%  Brent R. Hickman and Timothy P. Hubbard
% hickmanbr@uchicago.edu and timothy.hubbard@colby.edu

% please cite our Journal of Applied Econometrics paper "Replacing Sample
% Trimming with Boundary Correction in Nonparametric Estimation of
% First-Price Auctions" if this code is helpful to you.

% Monte Carlo using mixtures of Betas

clear all
clc
tic

% parameters for figure properties
lwidth = 2;
fsize = 14;
set(0,'defaulttextinterpreter','latex','Defaulttextfontsize',fsize);

% switch to save figures and workspace for a run
saverun = 1;
% switch to run mise analysis
runmise = 1;
% switch to run stage 1 (bid density) comparisons
runstageone = 1;
% make plots conditional on high pseudo value under respective estimator
conditional = 1;
% use hardle bandwidth transformation constant for non-gaussian kernel
usehardle = 1;
usesilver = 0;
usebc = 1;
usetrim = 0;

% number of players, number of auctions econometrician observes, and number
% of simulations
N = 5;
T = 200;
S = 1000;

% parameters of distributions
vlow = 0;
vhigh = 1;
aparm = [2 2 1 2];
bparm = [2 1 3 4];
gamma = 0.05;
ndists = length(aparm);
quadtol = 1e-8;

% set seed
rand('state',819);

% choice of kernel---everything else is automated
kernel = 'triweight';

% b0 is needed for h0 bandwidth---see KZ (2008) bottom of page 501 and the
% K0 term in (4.9) of ZK (1998) which is K0 = 6 + 18t + 12t^2, t\in[-1,0];
% this will compute b0 for any kernel choice provided by user
num_t1 = quad(@evalkspdf_num,-1,1,1e-14,[],kernel);
num_t2 = 0 - (-144/5 + 432/4 - 468/3 + 216/2 - 36);
denom_t1 = 0 - (-2 + 18/4 - 12/5);
denom_t2 = quad(@evalkspdf_denom,-1,1,1e-14,[],kernel);
b0 = ((num_t1^2*num_t2)/(denom_t1^2*denom_t2))^(1/5);

% valuation points to evaluate each estimator at, 500 just like GPV
neval = 500;
evalpts = linspace(vlow,vhigh,neval);

options = optimset('Display','off','GradObj','off','Hessian','off', ...
	'TolFun',1e-16,'TolX',1e-16);
if saverun == 1
	diary('cmdwinout.txt')
end

matlabpool open 4
for z=1:ndists

if saverun == 1
	fname = strcat('beta_mc_ex',num2str(z));
end
	
az = aparm(z);
bz = bparm(z);
	
% bid function at a grid of points
ngrid = 100000;
vgrid = linspace(vlow,vhigh,ngrid);
Fgrid = betacdf(vgrid,az,bz);
Ftrgrid = gamma*vgrid + (1 - gamma)*Fgrid;
Q = zeros(size(vgrid));
for i=2:ngrid
	Q(i) = quad('risamint_mixbetas',vgrid(i-1),vgrid(i),1e-14,[], ...
		az,bz,gamma,N);
	Q(i) = Q(i) + Q(i-1);
end
bgrid = vgrid - Q./(Ftrgrid.^(N - 1));
bgrid(1) = vgrid(1);
bhigh = max(bgrid);
%figure
%set(gcf,'DefaultLineLineWidth',lwidth)
%set(gca,'FontSize',fsize)
%box on
%plot(vgrid,bgrid,vgrid,vgrid)
%xlabel('$\mathbf{v}$')
%ylabel('$\mathbf{b}$')
%axis tight
%title(['Bid Function: Ex ',num2str(z)])
%if saverun == 1
%	fnameeps = strcat(fname,'.eps');
%	print('-depsc',fnameeps)
%end


% matrices of boundary-corrected and no boundary corrected valuation
% densities
b_evalpts = interp1(vgrid,bgrid,evalpts,'pchip');
neval = length(evalpts);
fV_bc_matrix = NaN*ones(S,neval);
fV_trimHH_matrix = NaN*ones(S,neval);
fV_trimSH_matrix = NaN*ones(S,neval);
if runstageone == 1
	gB_bc_matrix = NaN*ones(S,neval);
	gB_trimHH_matrix = NaN*ones(S,neval);
	gB_trimSH_matrix = NaN*ones(S,neval);
end

% matrices of bids, true, and estimated pseudo values
b_mat = NaN*ones(S,N*T);
v_true_mat = NaN*ones(S,N*T);
v_bc_mat = NaN*ones(S,N*T);
v_trimHH_mat = NaN*ones(S,N*T);
v_trimSH_mat = NaN*ones(S,N*T);
v_HH_mat = NaN*ones(S,N*T);
v_SH_mat = NaN*ones(S,N*T);

% random uniforms for all simulations so that it's outside parallel loops
% which involve different processors
umat = rand(S,N*T);

parfor s=1:S
u=[];
v=[]; 
b=[];
gB_bc=[];
gB_bc_right=[];
GB=[];
v_bc=[];
gB_trimHH=[];
gB_trimSH=[];
v_trimHH=[];
v_trimSH=[];
fV_bc=[];
fV_bc_right=[];
v_no_bc_trimmedHH=[];
v_no_bc_trimmedSH=[];
fV_trimHH=[];
fV_trimSH=[];
	u = umat(s,:);
	% need to find root of equation to get valuations corresponding to
	% uniform draws from mixture of Betas
	for i=1:N*T
		[v(i),fval,flag] = fzero('mixbetarnd',[0 1],options, ...
			u(i),az,bz,gamma);
		if flag == 1
			if (v(i) < vlow)
				fprintf('\nYikes! Random realization are less than vlow for example %d, simluation %d, draw %d\n',z,s,i)
			end
			if (v(i) > vhigh)
				fprintf('\nYikes! Random realization is greater than vhigh for example %d, simluation %d, draw %d\n',z,s,i)
			end
			if (imag(v(i)) > 0)
				fprintf('\nYikes! Random realization is imaginary for example %d, simluation %d, draw %d\n',z,s,i)
			end
		end
	end
	v = v';
	v_true_mat(s,:) = v;
	vmaxobs(s) = max(v);
    vminobs(s) = min(v);

	% transform valuations to bids
	b = interp1(vgrid,bgrid,v,'pchip');
	b_mat(s,:) = b;
	% some checks
	if (sum(isfinite(b)) < N*T)
		warning('Infinite bids')
	elseif (sum(isnan(b)) > 0)
		warning('NaN bids')
	elseif (max(b) > bhigh)
		warning('Bid exceeds maximum bid')
	end
	bmin(s) = min(b);
	bmax(s) = max(b);
		
	% step 1 of GPV: recover pseudo-values using boundary correction on
	% left boundary (and get "interior" points)
	[gB_bc,hb_l(s)] = kspdf_bc(b,kernel,b,b0,usehardle);
	% right-boundary correction: reflect bids over zero and do
	% left-boundary correction
	[gB_bc_right,hb_r(s)] = kspdf_bc(-b,kernel,-b,b0,usehardle);
	bind = find(b >= bmax(s) - hb_r(s));
	gB_bc(bind) = gB_bc_right(bind);
	GB = kscdf(b,'edf',b);
	v_bc = b' + GB./(gB_bc*(N - 1));
	v_bc = v_bc';
	v_bc_mat(s,:) = v_bc;
	vmin_bc(s) = min(v_bc);
	vmax_bc(s) = max(v_bc);
	% recover pseudo-values using no boundary correction (trimming HH)
	[gB_trimHH,hg_trimHH(s)] = kspdf_no_bc(b,kernel,b,N*T,usehardle);
	v_trimHH = b' + GB./(gB_trimHH*(N - 1));
    v_trimHH = v_trimHH';
    v_HH_mat(s,:) = v_trimHH;
    vmin_no_bcHH(s) = min(v_trimHH);
    vmax_no_bcHH(s) = max(v_trimHH);
   	% recover pseudo-values using no boundary correction (trimming SH)
	[gB_trimSH,hg_trimSH(s)] = kspdf_no_bc(b,kernel,b,N*T,usesilver);
	v_trimSH = b' + GB./(gB_trimSH*(N - 1));
    v_trimSH = v_trimSH';
    v_SH_mat(s,:) = v_trimSH;
	vmin_no_bcSH(s) = min(v_trimSH);
	vmax_no_bcSH(s) = max(v_trimSH);
        
    % need pseudo value at lower and upper trim points HH
	gB_trimptsHH = kspdf_no_bc(b,kernel,[bmin(s)+hg_trimHH(s) bmax(s)-hg_trimHH(s)], ...
		N*T,usehardle);
	GB_trimptsHH = kscdf(b,'edf',[bmin(s)+hg_trimHH(s) bmax(s)-hg_trimHH(s)]);
	lowlineHH(s) = bmin(s) + hg_trimHH(s) + GB_trimptsHH(1)/(gB_trimptsHH(1)*(N - 1));
	hilineHH(s) = bmax(s) - hg_trimHH(s) + GB_trimptsHH(2)/(gB_trimptsHH(2)*(N - 1));
	% need pseudo value at lower and upper trim points SH
	gB_trimptsSH = kspdf_no_bc(b,kernel,[bmin(s)+hg_trimSH(s) bmax(s)-hg_trimSH(s)], ...
		N*T,usesilver);
	GB_trimptsSH = kscdf(b,'edf',[bmin(s)+hg_trimSH(s) bmax(s)-hg_trimSH(s)]);
	lowlineSH(s) = bmin(s) + hg_trimSH(s) + GB_trimptsSH(1)/(gB_trimptsSH(1)*(N - 1));
	hilineSH(s) = bmax(s) - hg_trimSH(s) + GB_trimptsSH(2)/(gB_trimptsSH(2)*(N - 1));
	
	if runstageone == 1
		% extend step 1 to consider estimation only on bids, using bids at
		% evalpts (uniform valuation grid)
		[gB_bc_grid] = kspdf_bc(b,kernel,b_evalpts,b0,usehardle);
		[gB_bc_right_grid] = kspdf_bc(-b,kernel,-b_evalpts,b0,usehardle);
		bind = find(b_evalpts >= bmax(s) - hb_r(s));
		gB_bc_grid(bind) = gB_bc_right_grid(bind);
		[gB_trimHH_grid] = kspdf_no_bc(b,kernel,b_evalpts,N*T,usehardle);
		[gB_trimSH_grid] = kspdf_no_bc(b,kernel,b_evalpts,N*T,usesilver);
		gB_bc_matrix(s,:) = gB_bc_grid;
		gB_trimHH_matrix(s,:) = gB_trimHH_grid;
		gB_trimSH_matrix(s,:) = gB_trimSH_grid;
	end
	
	
	% step 2 of GPV: recover valuation densities using boundary correction
	% so no trimming is needed
	[fV_bc,hv_l(s)] = kspdf_bc(v_bc,kernel,evalpts,b0,usehardle);
	% right-boundary correction: reflect pseudo-valuations over zero and do
	% left-boundary correction
	[fV_bc_right,hv_r(s)] = kspdf_bc(-v_bc,kernel,-evalpts,b0,usehardle);
	vind = find(evalpts >= vmax_bc(s) - hv_r(s));
	fV_bc(vind) = fV_bc_right(vind);
	% recover pseudo-values using trimming---standard GPV with HH
	ind = [b v_trimHH];
	keep = [];
	[keep,ntrimHH(s)] = trim(ind,1,hg_trimHH(s));
	v_no_bc_trimmedHH = keep(:,2);
	vmax_trimHH(s) = max(v_no_bc_trimmedHH);
	vmin_trimHH(s) = min(v_no_bc_trimmedHH);
	[fV_trimHH,hfHH(s)] = kspdf_no_bc(v_no_bc_trimmedHH,kernel,evalpts, ...
		N*T,usehardle);
	% recover pseudo-values using trimming---standard GPV with SH
	ind = [];
	ind = [b v_trimSH];
	keep = [];
	[keep,ntrimSH(s)] = trim(ind,1,hg_trimSH(s));
	v_no_bc_trimmedSH = keep(:,2);
	vmax_trimSH(s) = max(v_no_bc_trimmedSH);
    vmin_trimSH(s) = min(v_no_bc_trimmedSH);
	[fV_trimSH,hfSH(s)] = kspdf_no_bc(v_no_bc_trimmedSH,kernel,evalpts, ...
		N*T,usehardle);
	% important stuff to keep
	fV_bc_matrix(s,:) = fV_bc;
	fV_trimHH_matrix(s,:) = fV_trimHH;
    fV_trimSH_matrix(s,:) = fV_trimSH;
    
    % compute mise over omega regions within simulations loops
    if runmise == 1
        % omega_HH
        intmin = vmin_trimHH(s) + hfHH(s);
        intmax = vmax_trimHH(s) - hfHH(s);
        gpvregion_ise_HH_HHreg(s) = quad('computeise',intmin,intmax,quadtol,[], ...
            v_no_bc_trimmedHH,az,bz,gamma,usetrim,kernel,b0,usehardle,N,T);
        gpvregion_ise_SH_HHreg(s) = quad('computeise',intmin,intmax,quadtol,[], ...
            v_no_bc_trimmedSH,az,bz,gamma,usetrim,kernel,b0,usehardle,N,T);
        gpvregion_ise_bc_HHreg(s) = quad('computeise',intmin,intmax,quadtol,[], ...
            v_bc,az,bz,gamma,usebc,kernel,b0,usehardle,N,T);
        % omega_SH
        intmin = vmin_trimSH(s) + hfSH(s);
        intmax = vmax_trimSH(s) - hfSH(s);
        gpvregion_ise_HH_SHreg(s) = quad('computeise',intmin,intmax,quadtol,[], ...
            v_no_bc_trimmedHH,az,bz,gamma,usetrim,kernel,b0,usehardle,N,T);
        gpvregion_ise_SH_SHreg(s) = quad('computeise',intmin,intmax,quadtol,[], ...
            v_no_bc_trimmedSH,az,bz,gamma,usetrim,kernel,b0,usehardle,N,T);
        gpvregion_ise_bc_SHreg(s) = quad('computeise',intmin,intmax,quadtol,[], ...
            v_bc,az,bz,gamma,usebc,kernel,b0,usehardle,N,T);
    end
    
end

% simple check that same right and left bandwiths using in BC
if (sum(hb_l == hb_r) ~= S) || (sum(hv_l == hv_r) ~= S)
	fprintf('Bandwidth issue at left and right boundaries\n')
end

% compute msep for each simulation and summary statistics
msep_bc = mean((v_bc_mat - v_true_mat).^2);
msep_trimHH = mean((v_HH_mat - v_true_mat).^2);
msep_trimSH = mean((v_SH_mat - v_true_mat).^2);
msep_bc_vmax = mean((vmaxobs - vmax_bc).^2);
msep_trimHH_vmax = mean((vmaxobs - vmax_trimHH).^2);
msep_trimSH_vmax = mean((vmaxobs - vmax_trimSH).^2);
msep_bc_vmin = mean((vminobs - vmin_bc).^2);
msep_trimHH_vmin = mean((vminobs - vmin_trimHH).^2);
msep_trimSH_vmin = mean((vminobs - vmin_trimSH).^2);


% mean over simulations of density estimates over grid space
funmix = betapdf(evalpts,az,bz);
ftrue = gamma*1 + (1 - gamma)*funmix;
Funmix = betacdf(evalpts,az,bz);
Ftrue = gamma*evalpts + (1 - gamma)*Funmix;

% aggregated all mise stuff here and it is run after simulations finish for
% a given instance
if runmise == 1
    if runstageone == 1
        funmix = betapdf(vgrid,az,bz);
        fgrid = gamma*1 + (1 - gamma)*funmix;
        Funmix = betacdf(vgrid,az,bz);
        Ftrue = gamma*vgrid + (1 - gamma)*Funmix;
        % compute bid density using g_B = f_V/sigma'(v)
        sigmaprime = (vgrid - bgrid).*(N - 1).*fgrid./Ftrue;
        sigmaprime(1) = sigmaprime(2) - (sigmaprime(3) - sigmaprime(2));
        gbgrid = fgrid./sigmaprime;
        
        minbmin = vlow;
        maxbmax = bhigh;
        % compute stage 1 mise
        stg1_mise_bc = quad('computeise_stg1_postrun',minbmin,maxbmax, ...
            quadtol,[],b_mat,bmin,bmax,usebc,kernel,b0,usehardle, ...
            bgrid,gbgrid);
        stg1_mise_trimHH = quad('computeise_stg1_postrun',minbmin,maxbmax, ...
            quadtol,[],b_mat,bmin,bmax,usetrim,kernel,b0,usehardle, ...
            bgrid,gbgrid);
        stg1_mise_trimSH = quad('computeise_stg1_postrun',minbmin,maxbmax, ...
            quadtol,[],b_mat,bmin,bmax,usetrim,kernel,b0,usesilver, ...
            bgrid,gbgrid);
    end
    
    minvminobs = vlow;
    maxvmaxobs = vhigh;
    mise_bc = quad('computeise_stg2_postrun',minvminobs,maxvmaxobs, ...
        quadtol,[],v_bc_mat,vmin_bc,vmax_bc,az,bz,gamma,usebc, ...
        kernel,b0,usehardle,N,T);
    % okay to pass v_HH_mat and not v_trimHH_mat as the vmin_trimHH and
    % vmax_trimHH will rule out trimmed values appropriately
    mise_trimHH = quad('computeise_stg2_postrun',minvminobs,maxvmaxobs, ...
        quadtol,[],v_HH_mat,vmin_trimHH,vmax_trimHH,az,bz,gamma,usetrim, ...
        kernel,b0,usehardle,N,T);
    % okay to pass v_HH_mat and not v_trimHH_mat as the vmin_trimHH and
    % vmax_trimHH will rule out trimmed values appropriately
    mise_trimSH = quad('computeise_stg2_postrun',minvminobs,maxvmaxobs, ...
        quadtol,[],v_SH_mat,vmin_trimSH,vmax_trimSH,az,bz,gamma,usetrim, ...
        kernel,b0,usehardle,N,T);
 
    % MISE over GPV regions
    % omega_HH
    gpvregion_mise_HH_HHreg = mean(gpvregion_ise_HH_HHreg);
    gpvregion_mise_SH_HHreg = mean(gpvregion_ise_SH_HHreg);
    gpvregion_mise_bc_HHreg = mean(gpvregion_ise_bc_HHreg);
    %omega_SH
    gpvregion_mise_HH_SHreg = mean(gpvregion_ise_HH_SHreg);
    gpvregion_mise_SH_SHreg = mean(gpvregion_ise_SH_SHreg);
    gpvregion_mise_bc_SHreg = mean(gpvregion_ise_bc_SHreg);
end


if runstageone == 1
    % first-stage estimation figures, compute bid density using g_B =
    % f_V/sigma'(v)
    Funmix = betacdf(evalpts,az,bz);
    Ftrue = gamma*evalpts + (1 - gamma)*Funmix;
    sigmaprime = (evalpts - b_evalpts).*(N - 1).*ftrue./Ftrue;
    gbtrue = ftrue./sigmaprime;
    if conditional == 1
        for i=1:neval
            ind = [];
            % see which simulations had random bids greater (or less) than
            % given grid point for conditional analysis
            if i >= neval/2
                ind = find(bmax >= b_evalpts(i));
            elseif i < neval/2
                ind = find(bmin <= b_evalpts(i));
            end
            gmean_bc(i) = mean(gB_bc_matrix(ind,i));
            gmed_bc(i) = median(gB_bc_matrix(ind,i));
            g5_bc(i) =  prctile(gB_bc_matrix(ind,i),5);
            g95_bc(i) =  prctile(gB_bc_matrix(ind,i),95);
            gmean_trimHH(i) = mean(gB_trimHH_matrix(ind,i));
            gmed_trimHH(i) = median(gB_trimHH_matrix(ind,i));
            g5_trimHH(i) =  prctile(gB_trimHH_matrix(ind,i),5);
            g95_trimHH(i) =  prctile(gB_trimHH_matrix(ind,i),95);
            gmean_trimSH(i) = mean(gB_trimSH_matrix(ind,i));
            gmed_trimSH(i) = median(gB_trimSH_matrix(ind,i));
            g5_trimSH(i) =  prctile(gB_trimSH_matrix(ind,i),5);
            g95_trimSH(i) =  prctile(gB_trimSH_matrix(ind,i),95);
        end
    elseif conditional == 0
        gmean_bc = mean(gB_bc_matrix);
        gmed_bc = median(gB_bc_matrix);
        g5_bc =  prctile(gB_bc_matrix,5);
        g95_bc =  prctile(gB_bc_matrix,95);
        gmean_trimHH = mean(gB_trimHH_matrix);
        gmed_trimHH = median(gB_trimHH_matrix);
        g5_trimHH = prctile(gB_trimHH_matrix,5);
        g95_trimHH = prctile(gB_trimHH_matrix,95);
        gmean_trimSH = mean(gB_trimSH_matrix);
        gmed_trimSH = median(gB_trimSH_matrix);
        g5_trimSH = prctile(gB_trimSH_matrix,5);
        g95_trimSH = prctile(gB_trimSH_matrix,95);
    end
end

% conditional on pseudo values existing at given grid point or not
if conditional == 1
    for i=1:neval
        ind = [];
        if i >= neval/2
            ind = find(vmax_bc >= evalpts(i));
        elseif i < neval/2
            ind = find(vmin_bc <= evalpts(i));
        end
        fmean_bc(i) = mean(fV_bc_matrix(ind,i));
        fmed_bc(i) = median(fV_bc_matrix(ind,i));
        f5_bc(i) =  prctile(fV_bc_matrix(ind,i),5);
        f95_bc(i) =  prctile(fV_bc_matrix(ind,i),95);
        ind = [];
        if i >= neval/2
            % the _no_bc values involve no trimming---they are the highest
            % and lowest implied pseudo values given bids without trimming;
            % the _trim values values are the highest and lowest pseudo
            % values from a given sample which are in the GPV region---the
            % highest and lowest implied pseudo values corresponding to a
            % sample of bids that are not within a bandwidth of the highest
            % and lowest pseudo values.  in contrast lowline (hiline) is
            % the lowest (highest) value defining the GPV region and it is
            % likely not observed in the data.
            ind = find(vmax_trimHH >= evalpts(i));
        elseif i < neval/2
            ind = find(vmin_trimHH <= evalpts(i));
        end
        fmean_trimHH(i) = mean(fV_trimHH_matrix(ind,i));
        fmed_trimHH(i) = median(fV_trimHH_matrix(ind,i));
        f5_trimHH(i) =  prctile(fV_trimHH_matrix(ind,i),5);
        f95_trimHH(i) =  prctile(fV_trimHH_matrix(ind,i),95);
        ind = [];
        if i >= neval/2
            ind = find(vmax_trimSH >= evalpts(i));
        elseif i < neval/2
            ind = find(vmin_trimSH <= evalpts(i));
        end
        fmean_trimSH(i) = mean(fV_trimSH_matrix(ind,i));
        fmed_trimSH(i) = median(fV_trimSH_matrix(ind,i));
        f5_trimSH(i) =  prctile(fV_trimSH_matrix(ind,i),5);
        f95_trimSH(i) =  prctile(fV_trimSH_matrix(ind,i),95);
    end
elseif conditional == 0
    fmean_bc = mean(fV_bc_matrix);
    fmed_bc = median(fV_bc_matrix);
    f5_bc =  prctile(fV_bc_matrix,5);
    f95_bc =  prctile(fV_bc_matrix,95);
    fmean_trimHH = mean(fV_trimHH_matrix);
    fmed_trimHH = median(fV_trimHH_matrix);
    f5_trimHH = prctile(fV_trimHH_matrix,5);
    f95_trimHH = prctile(fV_trimHH_matrix,95);
    fmean_trimSH = mean(fV_trimSH_matrix);
    fmed_trimSH = median(fV_trimSH_matrix);
	f5_trimSH = prctile(fV_trimSH_matrix,5);
	f95_trimSH = prctile(fV_trimSH_matrix,95);
end

% save workspace
if saverun == 1
	save(fname)
end

toc

end
matlabpool close
fprintf('Total run time is %.2f minutes\n',toc/60)